@@ -60,9 +60,9 @@
 #include "gpm-tray-icon.h"
 #include "gpm-engine.h"
 #include "gpm-upower.h"
-#include "gpm-disks.h"
 
 #include "org.mate.PowerManager.Backlight.h"
+#include "org.mate.PowerManager.KbdBacklight.h"
 
 static void     gpm_manager_finalize	(GObject	 *object);
 
@@ -77,7 +77,6 @@ struct GpmManagerPrivate
 {
 	GpmButton		*button;
 	GSettings		*settings;
-	GpmDisks		*disks;
 	GpmDpms			*dpms;
 	GpmIdle			*idle;
 	GpmControl		*control;
@@ -944,28 +943,6 @@ gpm_manager_button_pressed_cb (GpmButton *button, const gchar *type, GpmManager
 		gpm_manager_update_lid_throttle (manager, FALSE);
 }
 
-/**
- * gpm_manager_get_spindown_timeout:
- **/
-static gint
-gpm_manager_get_spindown_timeout (GpmManager *manager)
-{
-	gboolean enabled;
-	gint timeout;
-
-	/* get policy */
-	if (!manager->priv->on_battery) {
-		enabled = g_settings_get_boolean (manager->priv->settings, GPM_SETTINGS_SPINDOWN_ENABLE_AC);
-		timeout = g_settings_get_int (manager->priv->settings, GPM_SETTINGS_SPINDOWN_TIMEOUT_AC);
-	} else {
-		enabled = g_settings_get_boolean (manager->priv->settings, GPM_SETTINGS_SPINDOWN_ENABLE_BATT);
-		timeout = g_settings_get_int (manager->priv->settings, GPM_SETTINGS_SPINDOWN_TIMEOUT_BATT);
-	}
-	if (!enabled)
-		timeout = 0;
-	return timeout;
-}
-
 /**
  * gpm_manager_client_changed_cb:
  **/
@@ -1015,10 +992,6 @@ gpm_manager_client_changed_cb (UpClient *client, GpmManager *manager)
 
 	egg_debug ("on_battery: %d", on_battery);
 
-	/* set disk spindown threshold */
-	timeout = gpm_manager_get_spindown_timeout (manager);
-	gpm_disks_set_spindown_timeout (manager->priv->disks, timeout);
-
 	gpm_manager_sync_policy_sleep (manager);
 
 	gpm_manager_update_ac_throttle (manager);
@@ -1382,7 +1355,6 @@ gpm_manager_engine_charge_low_cb (GpmEngine *engine, UpDevice *device, GpmManage
 		/* TRANSLATORS: tell user more details */
 		message = g_strdup_printf (_("Cell phone is low in power (%.0f%%)"), percentage);
 
-#if UP_CHECK_VERSION(0,9,5)
 	} else if (kind == UP_DEVICE_KIND_MEDIA_PLAYER) {
 		/* TRANSLATORS: media player, e.g. mp3 is getting a little low */
 		title = _("Media player battery low");
@@ -1403,7 +1375,6 @@ gpm_manager_engine_charge_low_cb (GpmEngine *engine, UpDevice *device, GpmManage
 
 		/* TRANSLATORS: tell user more details */
 		message = g_strdup_printf (_("Attached computer is low in power (%.0f%%)"), percentage);
-#endif
 	}
 
 	/* get correct icon */
@@ -1526,7 +1497,6 @@ gpm_manager_engine_charge_critical_cb (GpmEngine *engine, UpDevice *device, GpmM
 					     "This device will soon stop functioning if not charged."),
 					   percentage);
 
-#if UP_CHECK_VERSION(0,9,5)
 	} else if (kind == UP_DEVICE_KIND_MEDIA_PLAYER) {
 
 		/* TRANSLATORS: the cell battery is very low */
@@ -1554,7 +1524,6 @@ gpm_manager_engine_charge_critical_cb (GpmEngine *engine, UpDevice *device, GpmM
 		message = g_strdup_printf (_("Attached computer is very low in power (%.0f%%). "
 					     "The device will soon shutdown if not charged."),
 					   percentage);
-#endif
 	}
 
 	/* get correct icon */
@@ -1672,8 +1641,10 @@ gpm_manager_engine_charge_action_cb (GpmEngine *engine, UpDevice *device, GpmMan
 	}
 
 	/* not all types have actions */
-	if (title == NULL)
+	if (title == NULL) {
+		g_free (message);
 		return;
+	}
 
 	/* get correct icon */
 	icon = gpm_upower_get_device_icon (device);
@@ -1851,7 +1822,6 @@ gpm_manager_init (GpmManager *manager)
 	manager->priv->notification_warning_low = NULL;
 	manager->priv->notification_discharging = NULL;
 	manager->priv->notification_fully_charged = NULL;
-	manager->priv->disks = gpm_disks_new ();
 	manager->priv->settings = g_settings_new (GPM_SETTINGS_SCHEMA);
 	g_signal_connect (manager->priv->settings, "changed",
 			  G_CALLBACK (gpm_manager_settings_changed_cb), manager);
@@ -1893,9 +1863,11 @@ gpm_manager_init (GpmManager *manager)
 
     manager->priv->kbd_backlight = gpm_kbd_backlight_new ();
     if (manager->priv->kbd_backlight != NULL) {
-        gpm_kbd_backlight_register_dbus (manager->priv->kbd_backlight,
-                                        g_connection,
-                                        NULL);
+    	dbus_g_object_type_install_info (GPM_TYPE_KBD_BACKLIGHT,
+						 &dbus_glib_gpm_kbd_backlight_object_info);
+		dbus_g_connection_register_g_object (connection, GPM_DBUS_PATH_KBD_BACKLIGHT,
+						     G_OBJECT (manager->priv->kbd_backlight));
+    
     }
 
 	manager->priv->idle = gpm_idle_new ();
@@ -1942,10 +1914,6 @@ gpm_manager_init (GpmManager *manager)
 	g_signal_connect (manager->priv->engine, "charge-action",
 			  G_CALLBACK (gpm_manager_engine_charge_action_cb), manager);
 
-	/* set disk spindown threshold */
-	timeout = gpm_manager_get_spindown_timeout (manager);
-	gpm_disks_set_spindown_timeout (manager->priv->disks, timeout);
-
 	/* update ac throttle */
 	gpm_manager_update_ac_throttle (manager);
 }
@@ -1981,7 +1949,6 @@ gpm_manager_finalize (GObject *object)
 		g_source_remove (manager->priv->critical_alert_timeout_id);
 
 	g_object_unref (manager->priv->settings);
-	g_object_unref (manager->priv->disks);
 	g_object_unref (manager->priv->dpms);
 	g_object_unref (manager->priv->idle);
 	g_object_unref (manager->priv->engine);
