@@ -505,7 +505,6 @@ gpm_device_kind_to_localised_text (UpDeviceKind kind, guint number)
 		/* TRANSLATORS: cell phone (mobile...) */
 		text = ngettext ("Cell phone", "Cell phones", number);
 		break;
-#if UP_CHECK_VERSION(0,9,5)
 	case UP_DEVICE_KIND_MEDIA_PLAYER:
 		/* TRANSLATORS: media player, mp3 etc */
 		text = ngettext ("Media player", "Media players", number);
@@ -518,7 +517,6 @@ gpm_device_kind_to_localised_text (UpDeviceKind kind, guint number)
 		/* TRANSLATORS: tablet device */
 		text = ngettext ("Computer", "Computers", number);
 		break;
-#endif
 	default:
 		egg_warning ("enum unrecognised: %i", kind);
 		text = up_device_kind_to_string (kind);
@@ -558,7 +556,6 @@ gpm_device_kind_to_icon (UpDeviceKind kind)
 	case UP_DEVICE_KIND_PHONE:
 		icon = "phone";
 		break;
-#if UP_CHECK_VERSION(0,9,5)
 	case UP_DEVICE_KIND_MEDIA_PLAYER:
 		icon = "multimedia-player";
 		break;
@@ -568,7 +565,6 @@ gpm_device_kind_to_icon (UpDeviceKind kind)
 	case UP_DEVICE_KIND_COMPUTER:
 		icon = "computer-apple-ipad";
 		break;
-#endif
 	default:
 		egg_warning ("enum unrecognised: %i", kind);
 		icon = "gtk-help";
@@ -652,6 +648,10 @@ gpm_device_state_to_localised_string (UpDeviceState state)
 		/* TRANSLATORS: battery state */
 		state_string = _("Waiting to discharge");
 		break;
+	case UP_DEVICE_STATE_UNKNOWN:
+		/* TRANSLATORS: battery state */
+		state_string = _("Unknown state");
+		break;
 	default:
 		g_assert_not_reached ();
 		break;
